/****************************************************************************
** $Id: qt/src/tools/qmemoryfile.cpp   2.3.4-beta2   edited 2002-09-18 $
**
** Implementation of the QMemoryFile class
**
** Created : 020813
**
** Copyright (C) 1992-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the tools module of the Qt GUI Toolkit.
**
** This file may be distributed under the terms of the Q Public License
** as defined by Trolltech AS of Norway and appearing in the file
** LICENSE.QPL included in the packaging of this file.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding valid Qt Enterprise Edition or Qt Professional Edition
** licenses may use this file in accordance with the Qt Commercial License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about Qt Commercial License Agreements.
** See http://www.trolltech.com/qpl/ for QPL licensing information.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/


#include "qmemoryfile_p.h"
#include "qfile.h"

/*****************************************************************************
  QMemoryFile member functions
 *****************************************************************************/

/*!
  \class QMemoryFile qmemoryfile_p.h
  \brief The QMemoryFile class provide means to map a file info memory or refere to a block or memory by name. 

  Warning: This class is not part of the Qt APi and subject to change. 
             This file file may change from version to version without 
             notice, or even be removed.


Currently this 
         class supports read only access on all platforms; and read plus write 
         accces provided on unix and windows based plaforms. 
	 The size of mapped file can not be changed.
*/


/*! \ enum QMemoryFile::Flags
  This enum specifies the possible types of connections made to the file to
  be memory mapped. 

   The currently defined values are : 

  \value Write  Allow write access to file.
  \value Shared Allow file to be shared with other proceses.
  \value Create - Create file named.
 */



/*!
  Construct a memory mapped to an actual file or named memory block.
  If \a fileName is not preceeded by an \\ then it is treated as a real 
     file name.
  Otherwise the characters after the \\ are used as the name for the memory block required.

  A value for \a size should always be provided, though if the size is not provided it will be determined if possible.

 If a file is being mapped to be written to, then the \a flags of subsequent QMemoryFiles mapped to the same file should include QMemoryFile::Write. 
  
 Example:
 \code 
 // Open a maping to file text.dat 
 QMemoryFile memoryFile("text.dat", QMemoryFile:Read, 20);
 char *data = (char*)memoryFile;
 int sum = 0;
 for (int i = 0; i < 20; i++){
   sum = sum + data[i];
 }
 qDebug("Sum =%d", sum);
 \endcode
 
 Example for creating named memory block:
 \code  

 QMemoryFile block1("\block1", QMemoryFile::Create | QMemoryFile::Write, 20);
 char *dataBlock = (char*)block;
 for (int i = 0; i < 19; i++){
   dataBlock[i] = i +'a';
 } 
 dataBlock[20] = '\0';
 qDebug("Data block is %s", dataBlock);
 \endcode

 
*/
QMemoryFile::QMemoryFile(const QString &fileName, int flags, uint size)
{
    block = NULL;
    length = 0;
    if (flags == -1)
      flags = QMemoryFile::Shared; // read only shared file mapping

    this->flags = flags;
    d = openData(fileName, flags, size);
}

/*!
  Destructs the memory mapped file 
*/
QMemoryFile::~QMemoryFile()
{
    closeData(d); 
}


/*!
  Returns true if memory is shared
*/
bool QMemoryFile::isShared() const
{
    return (flags & QMemoryFile::Shared) != 0;
}


/*! 
 Returns true if memory is writeable  
*/

bool QMemoryFile::isWriteable() const
{
    return (flags & QMemoryFile::Write) != 0;    
}



#if QT_PSEUDO_MMAP 

class QMemoryFileDataQMemoryFile::Write | QMemoryFile::Shared;
{ 
public:
    QMemoryFileData(QFile f, char* data, uint length);
    ~QMemoryFileData();
    operator char*() { return data;}

private:
    QFile f;
    uint length;
    char* data;
};


/*!
  Contructs the memory map data 
 */
QMemoryFileData::QMememoryFileData(QFile f, char *data, uint length )
{
    this->f = f;
    this->data = data;
    this->length = length;
}


/*!
  Destructs the memory map data
 */
QMemoryFileData::~QMemoryFileData()
{
    if (data != NULL){
	if (f.open(IO_Raw | IO_WriteOnly)){
	    f.writeBlock(data, length);
	}else
	    qDebug("QMemoryFile: Unable to write memory file back to disk");
	delete data;
    }
}



/*!
  \internal
  Initialize internal data.
  Returns non-null if able to intialize connection to file 
 */
QMemoryFileData *QMemoryFile::init(QString &fileName, int flags, uint size)
{
    QFile f(fileName);
    if (f.open(IO_Raw | IO_WriteOnly)){
	block = new char[f.size()];
	if ((block != NULL) && f.exists()){
	    uint fileSize = f.size();
	    if (f.readBlock(block, fileSize) == -1){
		delete block;
		block = NULL;
	    }else{
		this->flags = f | QMemoryFile::Shared;
		this->length = fileSize;
	    }	
	}
    }
    if (block != NULL)
#if !defined(_OS_WIN32_) 
	    qFatal("QMemoryFile: Failed to mmap %s",
		   QFile::encodeName(fileName).data());
#else
	    qFatal("QMemoryFile: Failed to mmap %s", 
		   (char*)fileName.latin1());
#endif	    	
    f.close();

    return new QMemoryFileData(f, block, fileSize); 
}


/*!
  \internal
  Close memory file and free any memory used
 */
void QMemoryFile::closeData(QMemoryFileData *memoryFile)
{
    delete memoryFile;
}

#endif
